/*
 * Copyright (C) 2016 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.fairphone.hiccup.app;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.text.Html;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;

public class HiccupUserConsentWizardActivity extends Activity implements View.OnClickListener, CompoundButton.OnCheckedChangeListener {

    private Button mButtonNext;

    private static final int UI_FLAGS = View.SYSTEM_UI_FLAG_LAYOUT_STABLE
            | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
            | View.SYSTEM_UI_FLAG_IMMERSIVE
            | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
            | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
    private View mRootView;
    private CheckBox mIWantToHelpCheckBox;

    public static void disable(Context ctx) {
        try {
            PackageManager localPackageManager = ctx.getPackageManager();
            localPackageManager.setComponentEnabledSetting(new ComponentName("com.fairphone.hiccup.app", "com.fairphone.hiccup.app.HiccupUserConsentWizardActivity"), PackageManager.COMPONENT_ENABLED_STATE_DISABLED, 1);
        } catch (Exception e) {
            HiccupUtil.LOG("Could not disable Wizard Component.");
        }
        HiccupUtil.LOG("Disabled setup wizard.\n");
    }

    public static void enable(Context ctx) {
        try {
            PackageManager localPackageManager = ctx.getPackageManager();
            localPackageManager.setComponentEnabledSetting(new ComponentName("com.fairphone.hiccup.app", "com.fairphone.hiccup.app.HiccupUserConsentWizardActivity"), PackageManager.COMPONENT_ENABLED_STATE_ENABLED, 1);
        } catch (Exception e) {
            HiccupUtil.LOG("Could not enable Wizard Component.");
        }
        HiccupUtil.LOG("Enabled setup wizard.\n");
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        final boolean isOwner = HiccupUtil.isOwner(this);

        if (!isOwner) {
            HiccupUserConsentWizardActivity.disable(this);
            finish();
        }

        final View decorView = getWindow().getDecorView();

        decorView.setSystemUiVisibility(UI_FLAGS);

        decorView.setOnSystemUiVisibilityChangeListener(
                new View.OnSystemUiVisibilityChangeListener() {

                    @Override
                    public void onSystemUiVisibilityChange(int visibility) {
                        if ((visibility & View.SYSTEM_UI_FLAG_FULLSCREEN) == 0) {
                            decorView.setSystemUiVisibility(UI_FLAGS);
                        }
                    }
                });

        setContentView(R.layout.consent_wizard);
        mRootView = findViewById(R.id.user_consent_wizard);
        mRootView.setSystemUiVisibility(UI_FLAGS);
        mButtonNext = (Button) findViewById(R.id.button_next);
        mButtonNext.setOnClickListener(this);

        mIWantToHelpCheckBox = (CheckBox) findViewById(R.id.iWantToHelpcheckBox);
        mIWantToHelpCheckBox.setOnCheckedChangeListener(this);

        TextView textViewMain = (TextView) findViewById(R.id.instructions);
        textViewMain.setText(Html.fromHtml(getString(R.string.wizard_consent_question)));
    }

    @Override
    protected void onRestart() {
        final View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(UI_FLAGS);
        super.onResume();
    }

    @Override
    protected void onResume() {
        final View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(UI_FLAGS);
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        HiccupSettings.setCrashReportingEnabled(this, mIWantToHelpCheckBox.isChecked());
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if ((keyCode == KeyEvent.KEYCODE_HOME)) {
            finish();
        }
        if ((keyCode == KeyEvent.KEYCODE_BACK)) {
            finish();
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public void onClick(View v) {
        if (v==mButtonNext) {
            HiccupUserConsentWizardActivity.disable(this);
            this.finish();
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (buttonView == mIWantToHelpCheckBox) {
            HiccupSettings.setCrashReportingEnabled(this, isChecked);
        }
    }
}
